@@ -67,6 +67,7 @@
 #include <sepol/policydb.h>
 #include <sepol/services.h>
 #include <sepol/conditional.h>
+#include <sepol/hierarchy.h>
 #include <sepol/flask.h>
 
 #include "queue.h"
@@ -103,10 +104,16 @@ void usage(char *progname)
 	exit(1);
 }
 
-static int print_sid(security_id_t sid,
+#define FGETS(out, size, in) \
+if (fgets(out,size,in)==NULL) {	\
+		fprintf(stderr, "fgets failed at line %d: %s\n", __LINE__,\
+				strerror(errno)); \
+			exit(1);\
+}
+static int print_sid(sepol_security_id_t sid,
 		     context_struct_t * context __attribute__ ((unused)), void *data __attribute__ ((unused)))
 {
-	security_context_t scontext;
+	sepol_security_context_t scontext;
 	size_t scontext_len;
 	int rc;
 
@@ -312,10 +319,10 @@ static void cond_check_type_rules(void)
 	}
 }
 
-extern char *av_to_string(uint32_t tclass, access_vector_t av);
+extern char *av_to_string(uint32_t tclass, sepol_access_vector_t av);
 
 void check_assertion_helper(unsigned int stype, unsigned int ttype, ebitmap_t *tclasses, 
-			    access_vector_t *avp, unsigned long line) 
+			    sepol_access_vector_t *avp, unsigned long line) 
 {
 	avtab_key_t avkey;
 	avtab_datum_t *avdatump;
@@ -444,19 +451,19 @@ int change_bool(char *name, int state)
 
 int main(int argc, char **argv)
 {
-	security_class_t tclass;
-	security_id_t ssid, tsid, *sids;
-	security_context_t scontext;
-	struct av_decision avd;
+	sepol_security_class_t tclass;
+	sepol_security_id_t ssid, tsid, *sids;
+	sepol_security_context_t scontext;
+	struct sepol_av_decision avd;
 	class_datum_t *cladatum;
-	char ans[80 + 1], *file = txtfile, *outfile = NULL, *path, *fstype;
+	char ans[80 + 1], *file = txtfile, *outfile = NULL, *path, *fstype, error_msg[ERRMSG_LEN];
 	size_t scontext_len, pathlen;
 	unsigned int i;
 	unsigned int protocol, port;
 	unsigned int binary = 0, debug = 0;
 	struct val_to_name v;
 	int ret, ch, fd;
-	unsigned int nel;
+	unsigned int nel, uret;
 	struct stat sb;
 	void *map;
 	FILE *outfp = NULL;
@@ -617,10 +624,16 @@ int main(int argc, char **argv)
 			fprintf(stderr, "%s:  policy lacks new netlink classes, unable to generate policy version %d\n", argv[0], policyvers);
 			exit(1);
 		}
-	    
+
+		if (hierarchy_check_constraints(&policydb, error_msg, sizeof(error_msg))) {
+			fprintf(stderr, "%s\n", error_msg);
+			exit(1);
+		}
+
 		/* remove type attributes */
 		hashtab_map_remove_on_error(policydb.p_types.table, 
 					    type_attr_remove, 0, 0);
+		
 		fclose(yyin);
 	}
 
@@ -685,19 +698,19 @@ int main(int argc, char **argv)
 	printf("q)  Exit\n");
 	while (1) {
 		printf("\nChoose:  ");
-		fgets(ans, sizeof(ans), stdin);
+		FGETS(ans, sizeof(ans), stdin);
 		switch (ans[0]) {
 		case '0':
 			printf("source sid?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ssid = atoi(ans);
 
 			printf("target sid?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			tsid = atoi(ans);
 
 			printf("target class?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			if (isdigit(ans[0])) {
 				tclass = atoi(ans);
 				if (!tclass || tclass > policydb.p_classes.nprim) {
@@ -749,7 +762,7 @@ int main(int argc, char **argv)
 			break;
 		case '1':
 			printf("sid?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ssid = atoi(ans);
 			ret = sepol_sid_to_context(ssid,
 					       &scontext, &scontext_len);
@@ -770,7 +783,7 @@ int main(int argc, char **argv)
 			break;
 		case '2':
 			printf("scontext?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			scontext_len = strlen(ans);
 			ans[scontext_len - 1] = 0;
 			ret = sepol_context_to_sid(ans, scontext_len,
@@ -795,14 +808,14 @@ int main(int argc, char **argv)
 			ch = ans[0];
 
 			printf("source sid?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ssid = atoi(ans);
 			printf("target sid?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			tsid = atoi(ans);
 
 			printf("object class?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			if (isdigit(ans[0])) {
 				tclass = atoi(ans);
 				if (!tclass || tclass > policydb.p_classes.nprim) {
@@ -845,7 +858,7 @@ int main(int argc, char **argv)
 			break;
 		case '7':
 			printf("pathname?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			pathlen = strlen(ans);
 			ans[pathlen - 1] = 0;
 			printf("%s:  loading policy configuration from %s\n", argv[0], ans);
@@ -883,7 +896,7 @@ int main(int argc, char **argv)
 			break;
 		case '8':
 			printf("fs kdevname?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 			sepol_fs_sid(ans, &ssid, &tsid);
 			printf("fs_sid %d default_file_sid %d\n",
@@ -891,7 +904,7 @@ int main(int argc, char **argv)
 			break;
 		case '9':
 			printf("protocol?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 			if (!strcmp(ans, "tcp") || !strcmp(ans, "TCP"))
 				protocol = IPPROTO_TCP;
@@ -902,14 +915,14 @@ int main(int argc, char **argv)
 				break;
 			}
 			printf("port? ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			port = atoi(ans);
 			sepol_port_sid(0, 0, protocol, port, &ssid);
 			printf("sid %d\n", ssid);
 			break;
 		case 'a':
 			printf("netif name?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 			sepol_netif_sid(ans, &ssid, &tsid);
 			printf("if_sid %d default_msg_sid %d\n",
@@ -922,7 +935,7 @@ int main(int argc, char **argv)
 			struct in6_addr addr6;
 			
 			printf("protocol family? ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 			if (!strcasecmp(ans, "ipv4"))
 				family = AF_INET;
@@ -934,7 +947,7 @@ int main(int argc, char **argv)
 			}
 			
 			printf("node address?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 			
 			if (family == AF_INET) {
@@ -956,10 +969,10 @@ int main(int argc, char **argv)
 		}	
 		case 'c':
 			printf("fstype?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
-			sepol_fs_use(ans, &ret, &ssid);
-			switch (ret) {
+			sepol_fs_use(ans, &uret, &ssid);
+			switch (uret) {
 			case SECURITY_FS_USE_XATTR:
 				printf("use xattr\n");
 				break;
@@ -980,15 +993,15 @@ int main(int argc, char **argv)
 			break;
 		case 'd':
 			printf("fstype?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 			fstype = strdup(ans);
 			printf("path?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 			path = strdup(ans);
 			printf("object class?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			if (isdigit(ans[0])) {
 				tclass = atoi(ans);
 				if (!tclass || tclass > policydb.p_classes.nprim) {
@@ -1012,12 +1025,12 @@ int main(int argc, char **argv)
 			break;
 		case 'e':
 			printf("from SID?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 			ssid = atoi(ans);
 
 			printf("username?  ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 
 			ret = sepol_get_user_sids(ssid, ans, &sids, &nel);
@@ -1048,7 +1061,7 @@ int main(int argc, char **argv)
 			break;
 		case 'h':
 			printf("name? ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 			
 			name = malloc((strlen(ans) + 1) * sizeof(char));
@@ -1060,7 +1073,7 @@ int main(int argc, char **argv)
 
 
 			printf("state? ");
-			fgets(ans, sizeof(ans), stdin);
+			FGETS(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 
 			if (atoi(ans))
