@@ -8,7 +8,7 @@ typedef struct te_assert {
 	ebitmap_t ttypes;
 	ebitmap_t tclasses;
         int self;
-	access_vector_t *avp;
+	sepol_access_vector_t *avp;
 	unsigned long line;
 	struct te_assert *next;
 } te_assert_t;
